@@ -207,7 +207,7 @@ class PsmPy:
             predicted_data_treatment = self.predicted_data[self.predicted_data[self.treatment] == 1]
             # return predicted_data
 
-    def knn_matched(self, matcher='propensity_logit', replacement=False, caliper=None):
+    def knn_matched(self, matcher='propensity_logit', replacement=False, caliper=None, drop_unmatched=True):
         """
         knn_matched -- Match data using k-nn algorithm
         Parameters
@@ -343,10 +343,10 @@ class PsmPy:
                     except:
                         indices_for_match.append(None)
 
-        if len(indexes[:, 1]) == len(np.unique(list(filter(None, indices_for_match)))):
+        if len(indexes[:, 1]) == len(list(filter(None, indices_for_match))):
             pass
         else:
-            warnings.warn('Some values do not have a match')
+            warnings.warn('Some values do not have a match. These are dropped for purposes of establishing a matched dataframe, and subsequent calculations and plots (effect size). If you do not wish this to be the case please set drop_unmatched=False')
 
         ID_match = []
         for idxxx in indices_for_match:
@@ -360,10 +360,22 @@ class PsmPy:
 
         indexes_nonull = list(filter(None, indices_for_match))
         major_matched = major_pred_rstindx.take(indexes_nonull)
-        self.df_matched = pd.concat(
-            [minor_pred_rstindx, major_matched], axis=0, ignore_index=True)
-        minor_pred_rstindx['matched_ID'] = ID_match
-        self.matched_ids = minor_pred_rstindx[[self.indx, 'matched_ID']]
+
+        if drop_unmatched == True:
+            minor_pred_rstindx['matched_ID'] = ID_match
+            self.matched_ids = minor_pred_rstindx[[self.indx, 'matched_ID']]
+            self.matched_ids = self.matched_ids.dropna().reset_index(drop=True)
+
+            ids_minor_cleaned = self.matched_ids[self.indx].astype(
+                int).tolist()
+            minor_pred_rstindx = minor_pred_rstindx.take(ids_minor_cleaned)
+            self.df_matched = pd.concat(
+                [minor_pred_rstindx, major_matched], axis=0, ignore_index=True)
+        else:
+            self.df_matched = pd.concat(
+                [minor_pred_rstindx, major_matched], axis=0, ignore_index=True)
+            minor_pred_rstindx['matched_ID'] = ID_match
+            self.matched_ids = minor_pred_rstindx[[self.indx, 'matched_ID']]
 
     def knn_matched_12n(self, matcher='propensity_logit', how_many=1):
         """
@@ -455,6 +467,7 @@ class PsmPy:
         df_2_extract_major_class = pd.DataFrame(ID_match, columns=[self.indx])
         df_4_user = pd.merge(
             major_pred_rstindx_og, df_2_extract_major_class, on=self.indx, how="right")
+
         self.df_matched = pd.concat(
             [minor_pred_rstindx, df_4_user], axis=0, ignore_index=True)
 
